// ===================================================================
// Magic cube scrambler for 3x3x3 Rubik cubes
// -------------------------------------------------------------------
// This is a simple MPL door for anyone who likes random scrambles for
// their magic cube to solve.
// -------------------------------------------------------------------
// Author:		Arjen Schumacher
// Board:		stealthworld.tk:2023 
// Site:		http://stealthworld.tk
// Internet radio:	http://stealthworld.tk:8000/hardmoon
// ===================================================================
  
Uses Cfg
Uses User

Const Version = '1.0'
Const Program = 'Magic Cube Scrambler'
Const Author = 'Onky'
Const Copyright = '2021'

Var inputVal : String;
Var scrambleSet : Array[1..20] of String;
Var cubeRot : Integer;
Var cubeSide : Integer;

//===================================
// Help page
//===================================
Procedure HelpPage
Begin
   Write('|CL|DF')
   WriteXY(5,2,15,'Help: What does this scrambler do?') 

   WriteXY(5,4,7,'This program will generate a row of codes to scramble a')
   WriteXY(5,5,7,'solved cube in 20 steps of side-rotations, so you can')
   WriteXY(5,6,7,'then try to solve the cube as fast as possible.')

   WriteXY(5,8,15,'How do I read these scramble codes?')

   WriteXY(5,10,7,'F = Front side')
   WriteXY(5,11,7,'R = Right side')
   WriteXY(5,12,7,'U = Upper side')

   WriteXY(34,10,7,'B = Back side')    
   WriteXY(34,11,7,'L = Left side')
   WriteXY(34,12,7,'D = Down side')
   
   WriteXY(5,14,7,'If the sign ` is behind a side letter (F` for instance),')

   WriteXY(5,15,7,'it means you need to rotate the front side counter-clockwise 90 degrees.')
   WriteXY(5,16,7,'If no ` sign is present, you need to turn that side clockwise 90 degrees.')
   WriteXY(5,18,7,'A digit 2 after a side letter (B2 for instance), means you')
   WriteXY(5,19,7,'turn Back side 2 times (or 180 degrees).') 
   WriteXY(5,24,15,'Understood!! [PRESS ENTER]')

   Input(0,0,15,'')
  
End

//===================================
// Show the generated scramble
//===================================
Procedure displayScramble
Var X : Integer;
Var scramble : String;
Begin
   If scrambleSet[1] <> '' Then
   Begin
      WriteXY(2,13,14,'')
      For X := 1 to 20 Do
      Begin
        scramble := scramble + scrambleSet[X]+'  '
      End
      WriteXY(40-((Length(scramble)/2)-2),13,15,scramble)
      For X := 1 to 78 Do
      Begin 
         WriteXY(X,14,9,#196)
         WriteXY(X,12,9,#196)
      End
   End
End


//===================================
// Just some text on refreshed screen
//===================================
Procedure WriteHeader
Var offsetX : Integer = 10;
Var offsetY : Integer = 2;
Begin
   Write('|CL|DF')
   WriteXY(offsetX,offsetY,14,Program)
   Write('|08 v'+Version)
   WriteXY(offsetX,offsetY+1,4,'Mix up your cube for your next solve!')
   WriteXY(offsetX,offsetY+2,8,'Written by '+Author+' from stealthworld.tk:2023')
   WriteXY(offsetX,offsetY+6,7,'Generate random scrambles to mix up a solved 3*3*3 cube.')

   displayScramble
 
   WriteXY(offsetX,offsetY+18,7,'[S] Scramble! - [H] Help - [Q] Quit')
End



//===================================
// Get user input 
//===================================
Procedure GetInput
Begin
      WriteXY(10,22,15,'Your command: ')
      inputVal := Input(1,1,2,'')
End

Procedure PickSide
Begin
  cubeSide := Random(6)
End

Procedure PickRotation
Begin
  cubeRot := Random(3)
End


//===================================
// Scramble routine
//===================================
Procedure Scramble

Var Finished : Boolean = False;
Var CorrectSide : Boolean = False;
Var Counter : Integer = 1;
Var T : String;
Var arrSides : Array[0..19] of Integer;
Begin
   While not Finished Do Begin
     
     CorrectSide := False
     While CorrectSide = False Do
     Begin
        PickSide
        PickRotation
     
	arrSides[Counter] := cubeSide

        If cubeSide = 0 Then
           T := 'F'
        Else If cubeSide = 1 Then
           T := 'B'
        Else If cubeSide = 2 Then
           T := 'L' 
        Else If cubeSide = 3 Then
           T := 'R'
        Else If cubeSide = 4 Then
           T := 'U'
        Else If cubeSide = 5 Then
           T := 'D'
     
        If cubeRot = 1 Then
           T := T + '`'
        Else If cubeRot = 2 Then
           T := T + '2'
        
        If Counter = 0 or arrSides[Counter-1] <> arrSides[Counter] Then
           CorrectSide := True
        If Counter > 1 and arrSides[Counter-2] = arrSides[Counter] Then
           CorrectSide := False
     End
         
     scrambleSet[Counter] := T
     //REMOVE: WriteLn(Int2Str(cubeSide)+' '+Int2Str(cubeRot)+': '+T)
     
     Counter:=Counter+1     
     If Counter >= 21 Then
        Finished := True
   End
End


//===================================
// Main loop
//===================================
Procedure Main
Var Finished : Boolean = False;
Begin
   While not Finished Do Begin
      WriteHeader
      GetInput      
      If inputVal = 'Q' Then
         Finished := True
      Else If inputVal = 'H' Then
         HelpPage
      Else If inputVal = 'S' Then
         Scramble
   End
End


//==================================
// Core procedure
//==================================
Begin 
   Main
End
